var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input, ElementRef, ComponentFactoryResolver, ViewContainerRef } from '@angular/core';
import { SuperTableState } from './SuperTableState';
import { forEach, values } from 'lodash';
var EnumFilterDropdown = (function () {
    function EnumFilterDropdown(state, el) {
        this.state = state;
        this.el = el;
    }
    EnumFilterDropdown.prototype.ngOnInit = function () {
        var styles = this.el.nativeElement.style;
        styles.top = this.top + 'px';
        styles.left = this.left + 'px';
        styles.width = this.width + 'px';
    };
    EnumFilterDropdown.prototype.ngOnDestroy = function () {
        // to ensure that references to parent component
        // do not prevent GC
        this.destroyMe = null;
    };
    EnumFilterDropdown.prototype.onChoiceChange = function () {
        this.state.notify();
    };
    EnumFilterDropdown.prototype.showAll = function () {
        var _this = this;
        forEach(this.column.filterValue, function (val, key) {
            _this.column.filterValue[key] = true;
        });
        this.state.notify();
        this.destroyMe();
    };
    return EnumFilterDropdown;
}());
__decorate([
    Input(),
    __metadata("design:type", Object)
], EnumFilterDropdown.prototype, "column", void 0);
EnumFilterDropdown = __decorate([
    Component({
        selector: 'super-table-enum-filter-dropdown',
        template: "\n    <div class=\"clear-filter\">\n      <button class=\"btn btn-secondary clear-filter-btn\" role=\"button\" (click)=\"showAll()\">show all</button>\n    </div>\n    <div *ngFor=\"let choice of column.def.filterChoices\">\n      <input type=\"checkbox\" [(ngModel)]=\"column.filterValue[choice]\" (ngModelChange)=\"onChoiceChange($event)\" />\n      {{ choice }}\n    </div>\n    <button role=\"button\" class=\"close-dropdown\" (click)=\"destroyMe()\">&times;</button>\n  ",
        styles: ["\n    :host {\n      position: absolute;\n      background: white;\n      padding: 5px 10px;\n      border: 1px solid #ddd;\n      box-shadow: 0 1px 10px -1px rgba(0,0,0,0.2);\n    }\n    .clear-filter {\n      border-bottom: 1px solid #DDD;\n      padding: 5px 0;\n    }\n    .close-dropdown {\n      position: absolute;\n      top: 5px;\n      right: 10px;\n      border: none;\n      background: transparent;\n      color: #CCC;\n      display: block;\n      width: 20px;\n      height: 20px;\n      line-height: 20px;\n    }\n    .close-dropdown:hover {\n      color: #AAA;\n    }\n  "]
    }),
    __metadata("design:paramtypes", [SuperTableState,
        ElementRef])
], EnumFilterDropdown);
export { EnumFilterDropdown };
var EnumFilter = (function () {
    function EnumFilter(state, el, viewContainer, resolver) {
        this.state = state;
        this.el = el;
        this.viewContainer = viewContainer;
        this.resolver = resolver;
        this.disabledChoices = new Set();
        this.disabledFilterCount = 0;
    }
    EnumFilter.prototype.ngOnInit = function () {
        var _this = this;
        // initialize filtered values to include all
        this.column.filterValue = {};
        this.column.def.filterChoices.forEach(function (choice) {
            _this.column.filterValue[choice] = true;
        });
        this.subscription = this.state.stateChanged$.subscribe(function () {
            _this.disabledFilterCount = values(_this.column.filterValue)
                .filter(function (isEnabled) { return !isEnabled; })
                .length;
        });
    };
    EnumFilter.prototype.ngOnDestroy = function () {
        this.subscription.unsubscribe();
    };
    EnumFilter.prototype.toggleVisibility = function () {
        var _this = this;
        if (this.dropdown) {
            this.dropdown.destroy();
            this.dropdown = null;
        }
        else {
            var clientRect = this.el.nativeElement.getBoundingClientRect();
            var cmpFactory = this.resolver.resolveComponentFactory(EnumFilterDropdown);
            var ctxInjector = this.viewContainer.injector;
            var cmpRef = this.viewContainer.createComponent(cmpFactory, 0, ctxInjector);
            cmpRef.instance.column = this.column;
            cmpRef.instance.top = clientRect.top + clientRect.height;
            cmpRef.instance.left = clientRect.left;
            cmpRef.instance.width = clientRect.width;
            cmpRef.instance.destroyMe = function () {
                _this.toggleVisibility();
            };
            this.dropdown = cmpRef;
            document.body.appendChild(cmpRef.location.nativeElement);
        }
    };
    return EnumFilter;
}());
__decorate([
    Input(),
    __metadata("design:type", Object)
], EnumFilter.prototype, "filter", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], EnumFilter.prototype, "column", void 0);
EnumFilter = __decorate([
    Component({
        selector: '[enum-filter]',
        template: "\n    <button\n      [attr.title]=\"filter.title\"\n      role=\"button\"\n      (click)=\"toggleVisibility($event)\"\n      [ngClass]=\"{ hasDisabled : disabledFilterCount }\">\n      <strong>{{ filter.placeholder }}:</strong>\n      <span [hidden]=\"disabledFilterCount != 0\">showing all</span>\n      <span [hidden]=\"disabledFilterCount == 0\">filtering {{disabledFilterCount}} value(s)</span>\n    </button>\n\n  ",
        styles: ["\n    :host {\n      position: relative;\n      display: block;\n      height: 100%;\n      font-size: 90%;\n      white-space: nowrap;\n      text-overflow: ellipsis;\n    }\n    button {\n      text-align: left;\n      margin: 0 5px;\n      display: block;\n      width: calc(100% - 10px);\n      height: 100%;\n      background: #ddd;\n      border: none;\n      border-radius: 2px;\n    }\n    button.hasDisabled {\n      background: rgba(22, 140, 239, 0.2);\n    }\n    .enum-filter-dropdown {\n      position: absolute;\n      top: 100%;\n      z-index: 1;\n    }\n  "]
    }),
    __metadata("design:paramtypes", [SuperTableState,
        ElementRef,
        ViewContainerRef,
        ComponentFactoryResolver])
], EnumFilter);
export { EnumFilter };
