var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input, ElementRef } from '@angular/core';
import { SuperTableState } from './SuperTableState';
var Resizer = Resizer_1 = (function () {
    function Resizer(el) {
        this.el = el;
    }
    Resizer.prototype.grab = function (grabEvt) {
        var _this = this;
        grabEvt.preventDefault();
        var mousedownTime = Date.now();
        var initClientX = grabEvt.clientX;
        var initWidth = this.column.width || this.getActualParentWidth();
        var drag = function (event) {
            var change = event.clientX - initClientX;
            _this.column.width = Math.max(initWidth + change, Resizer_1.MIN_COLUMN_WIDTH);
        };
        var unbindDrag = function () {
            window.removeEventListener('mousemove', drag);
            window.removeEventListener('mouseup', unbindDrag);
            if (Date.now() - mousedownTime < Resizer_1.MAX_CLICK_WAIT) {
                _this.column.width = null;
            }
        };
        window.addEventListener('mousemove', drag);
        window.addEventListener('mouseup', unbindDrag);
    };
    Resizer.prototype.getActualParentWidth = function () {
        return this.el.nativeElement.parentElement.offsetWidth;
    };
    Resizer.prototype.stopClick = function (event) {
        event.preventDefault();
        event.stopPropagation();
    };
    return Resizer;
}());
Resizer.MAX_CLICK_WAIT = 250;
Resizer.MIN_COLUMN_WIDTH = 30;
__decorate([
    Input(),
    __metadata("design:type", Object)
], Resizer.prototype, "column", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], Resizer.prototype, "actualWidth", void 0);
Resizer = Resizer_1 = __decorate([
    Component({
        selector: '[resizer]',
        template: "<div class=\"notch\" [ngClass]=\"{ explicit: column.width }\"></div>",
        host: {
            '(click)': 'stopClick($event)',
            '(mousedown)': 'grab($event)',
            '[attr.title]': '"Click-and-drag to resize. Click to clear specified width."'
        },
        styles: ["\n    :host {\n      position: absolute;\n      right: 0;\n      top: 0;\n      width: 5px;\n      height: 100%;\n      cursor: col-resize;\n    }\n    .notch.explicit {\n      background-color: rgba(22, 140, 239, 0.2);\n    }\n    .notch {\n      width: 100%;\n      height: 50%;\n      transform: translateY(50%);\n      box-shadow: inset 1px 0 #DDD;\n    }\n  "]
    }),
    __metadata("design:paramtypes", [ElementRef])
], Resizer);
export { Resizer };
var TableHeader = (function () {
    function TableHeader(el, state) {
        this.el = el;
        this.state = state;
        this.noHeight = false;
        this.SORT_TITLE = 'Click to change sort order. Shift-click to sort on multiple columns.';
    }
    TableHeader.prototype.getWidth = function () {
        return (typeof this.column.width === 'number') ? this.column.width + 'px' : 'auto';
    };
    TableHeader.prototype.getValue = function () {
        return this.column.def.label;
    };
    TableHeader.prototype.handleClick = function (event) {
        event.preventDefault();
        if (this.column.hasSort) {
            this.state.toggleSort(this.column, event.shiftKey);
        }
    };
    return TableHeader;
}());
__decorate([
    Input(),
    __metadata("design:type", Object)
], TableHeader.prototype, "column", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], TableHeader.prototype, "noHeight", void 0);
TableHeader = __decorate([
    Component({
        selector: '[table-header]',
        host: {
            '[style.width]': 'getWidth()',
            '(click)': 'handleClick($event)'
        },
        template: "\n    <div *ngIf=\"!noHeight\" class=\"table-header-div\" title=\"SORT_TITLE\">\n      <span *ngIf=\"column.def.sort\" class=\"sort-icon\">\n        <span [ngSwitch]=\"column.sortOrder\">\n          <span class=\"asc-sort glyphicon glyphicon-sort-by-attributes\" *ngSwitchCase=\"'ASC'\"></span>\n          <span class=\"desc-sort glyphicon glyphicon-sort-by-attributes-alt\" *ngSwitchCase=\"'DESC'\"></span>\n          <span class=\"no-sort glyphicon glyphicon-sort\" *ngSwitchDefault></span>\n        </span>\n      </span>\n      {{ getValue() }}\n    </div>\n    <div *ngIf=\"!noHeight && !column.def.lockWidth\" resizer [column]=\"column\"></div>\n  ",
        styles: ["\n    :host {\n      position:relative;\n      -webkit-touch-callout: none;\n      -webkit-user-select: none;\n      -khtml-user-select: none;\n      -moz-user-select: none;\n      -ms-user-select: none;\n      user-select: none;\n    }\n    :host:hover .sort-icon {\n      opacity: 1;\n    }\n    .table-header-div {\n      position: relative;\n    }\n    .sort-icon {\n      font-size: 70%;\n      opacity: 1;\n      color: #168cef;\n      text-shadow: 0 1px 2px rgba(22, 140, 239, 0.6);\n    }\n    .sort-icon .no-sort {\n      opacity: 0.3;\n      text-shadow: none;\n      color: black;\n    }\n  "]
    }),
    __metadata("design:paramtypes", [ElementRef, SuperTableState])
], TableHeader);
export { TableHeader };
var Resizer_1;
